@@ -51,7 +51,6 @@
 #include "a11y/htmlboxembeddedaccessible.h"
 #include "a11y/htmlboxaccessible.h"
 #include "a11y/htmlboxtableaccessible.h"
-#include "a11y/htmlboxtextaccessible.h"
 #endif
 
 enum {
@@ -82,7 +81,6 @@ HTML_ACCESSIBLE_FACTORY (HTML_TYPE_BOX_BLOCK_ACCESSIBLE, html_box_block_accessib
 HTML_ACCESSIBLE_FACTORY (HTML_TYPE_BOX_EMBEDDED_ACCESSIBLE, html_box_embedded_accessible)
 HTML_ACCESSIBLE_FACTORY (HTML_TYPE_BOX_ACCESSIBLE, html_box_accessible)
 HTML_ACCESSIBLE_FACTORY (HTML_TYPE_BOX_TABLE_ACCESSIBLE, html_box_table_accessible)
-HTML_ACCESSIBLE_FACTORY (HTML_TYPE_BOX_TEXT_ACCESSIBLE, html_box_text_accessible)
 #endif
 
 
@@ -215,11 +213,11 @@ html_view_relayout (HtmlView *view)
 			html_view_try_jump (view);
 	}
 	if (view->relayout_timeout_id != 0) {
-		gtk_timeout_remove (view->relayout_timeout_id);
+		g_source_remove (view->relayout_timeout_id);
 		view->relayout_timeout_id = 0;
 	}
 	if (view->relayout_idle_id != 0) {
-		gtk_idle_remove (view->relayout_idle_id);
+		g_source_remove (view->relayout_idle_id);
 		view->relayout_idle_id = 0;
 	}
 
@@ -241,7 +239,7 @@ relayout_timeout_callback (gpointer data)
 	view->relayout_timeout_id = 0;
 
 	if (view->relayout_idle_id != 0) {
-		gtk_idle_remove (view->relayout_idle_id);
+		g_source_remove (view->relayout_idle_id);
 		view->relayout_idle_id = 0;
 	}
         return FALSE;
@@ -258,7 +256,7 @@ relayout_idle_callback (gpointer data)
 	view->relayout_idle_id = 0;
 
 	if (view->relayout_timeout_id != 0) {
-		gtk_timeout_remove (view->relayout_timeout_id);
+		g_source_remove (view->relayout_timeout_id);
 		view->relayout_timeout_id = 0;
 	}
         return FALSE;
@@ -268,14 +266,14 @@ static void
 html_view_relayout_when_idle (HtmlView *view)
 {
 	if (view->relayout_idle_id == 0)
-		view->relayout_idle_id = gtk_idle_add (relayout_idle_callback, view);
+		view->relayout_idle_id = g_idle_add (relayout_idle_callback, view);
 }
 
 static void
 html_view_relayout_after_timeout (HtmlView *view)
 {
 	if (view->relayout_timeout_id == 0)
-		view->relayout_timeout_id = gtk_timeout_add (RELAYOUT_TIMEOUT_INTERVAL, relayout_timeout_callback, view);
+		view->relayout_timeout_id = g_timeout_add (RELAYOUT_TIMEOUT_INTERVAL, relayout_timeout_callback, view);
 }
 
 static void
@@ -649,6 +647,8 @@ static void
 html_view_realize (GtkWidget *widget)
 {
 	HtmlView *view;
+	gfloat fsize;
+	gint isize;
 
 	view = HTML_VIEW (widget);
 
@@ -659,6 +659,15 @@ html_view_realize (GtkWidget *widget)
 	widget->style = gtk_style_copy (widget->style);
 	widget->style->bg[GTK_STATE_NORMAL] = 
 		widget->style->base[GTK_STATE_NORMAL];
+	/*
+	 * Store the font size so we can adjust size of HtmlFontSpecification
+	 * if the size changes.
+	 */
+	fsize = pango_font_description_get_size (widget->style->font_desc) / 
+			(gfloat) PANGO_SCALE;
+	isize = (gint) fsize;
+	g_object_set_data (G_OBJECT (widget), "html-view-font-size",
+			   GINT_TO_POINTER (isize));
 	
 	if (GTK_WIDGET_CLASS (parent_class)->realize)
 		(* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
@@ -808,12 +817,12 @@ html_view_destroy (GtkObject *object)
 	gpointer saved_focus;
 
 	if (view->relayout_timeout_id != 0) {
-		gtk_timeout_remove (view->relayout_timeout_id);
+		g_source_remove (view->relayout_timeout_id);
 		view->relayout_timeout_id = 0;
 	}
 
 	if (view->relayout_idle_id != 0) {
-		gtk_idle_remove (view->relayout_idle_id);
+		g_source_remove (view->relayout_idle_id);
 		view->relayout_idle_id = 0;
 	}
 
@@ -1270,12 +1279,88 @@ html_view_get_type (void)
 	return type;
 }
 
-void
+static void 
+html_view_update_box_style_size (HtmlBox *root, gfloat adjust, GPtrArray *done)
+{
+	HtmlBox *box;
+
+	for (box = root; box; box = box->next) {
+		HtmlStyle *style;
+
+		style = HTML_BOX_GET_STYLE (box);
+		if (style) {
+			HtmlFontSpecification *font_spec;
+
+			font_spec = style->inherited->font_spec;
+			if (font_spec) {
+				gint i;
+				gboolean found = FALSE;
+
+				for (i = 0; i < done->len; i++) {
+					if (g_ptr_array_index (done, i) == font_spec) {
+						found = TRUE;
+						break;
+					}
+				}
+				if (!found) {
+					g_ptr_array_add (done, font_spec);
+					font_spec->size = font_spec->size * adjust;
+				}
+			}
+		}
+		if (box->children) {
+			html_view_update_box_style_size (box->children, adjust, done);
+		}
+	}
+}
+
+static void
 html_view_style_set (GtkWidget *widget, GtkStyle *previous_style)
 {
 	if (previous_style) {
+		gfloat fsize;
+		gint new_isize;
+		gint old_isize;
+
  		widget->style->bg[GTK_STATE_NORMAL] =
  			widget->style->base[GTK_STATE_NORMAL];
+		fsize = pango_font_description_get_size (widget->style->font_desc) / (gfloat) PANGO_SCALE;
+		new_isize = (gint) fsize;
+		old_isize = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "html-view-font-size"));
+		if (old_isize && old_isize != new_isize) {
+			/*
+			 * The font size for the HtmlView has changed so we
+			 * update the size in each HtmlFontSpecification.
+			 * We traverse the document but we only want to
+			 * update the HtmlFontSpecification once. We maintain
+			 * an array of HtmlFontSpecification which have already
+			 * been updated. As a HtmlFontSpecification may be used
+			 * more than than one HtmlView we use a static array
+			 * which is cleared when we determine that a different
+			 * size change has occurred.
+			 */
+			HtmlView *view;
+
+			view = HTML_VIEW (widget);
+			g_object_set_data (G_OBJECT (widget), "html-view-font-size",
+					   GINT_TO_POINTER (new_isize));
+			if (view->root) {
+				static gint old_size = 0;
+				static gint new_size = 0;
+				static GPtrArray *done;
+
+				if (old_size != old_isize ||
+				    new_size != new_isize) {
+					if (old_size || new_size) {
+						g_ptr_array_free (done, TRUE);
+					}
+					done = g_ptr_array_new ();
+					old_size = old_isize;
+					new_size = new_isize;
+				}
+				html_view_update_box_style_size (view->root, (gfloat) new_isize / (gfloat) old_isize, done);
+			}
+		}
 	}
 }
 
@@ -1386,7 +1471,6 @@ html_view_get_accessible (GtkWidget *widget)
 			HTML_ACCESSIBLE_SET_FACTORY (HTML_TYPE_BOX_EMBEDDED, html_box_embedded_accessible);
 			HTML_ACCESSIBLE_SET_FACTORY (HTML_TYPE_BOX, html_box_accessible);
 			HTML_ACCESSIBLE_SET_FACTORY (HTML_TYPE_BOX_TABLE, html_box_table_accessible);
-			HTML_ACCESSIBLE_SET_FACTORY (HTML_TYPE_BOX_TEXT, html_box_text_accessible);
 		}
 		first_time = FALSE;
 	}
